/* Copyright 2014-2017 Research Foundation State University of New York   */

/* This file is part of QUB Online.                                       */

/* QUB Online is free software; you can redistribute it and/or modify    */
/* it under the terms of the GNU General Public License as published by   */
/* the Free Software Foundation, either version 3 of the License, or      */
/* (at your option) any later version.                                    */

/* QUB Online is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License,     */
/* named LICENSE.txt, in the QUB Online program directory.  If not, see  */
/* <http://www.gnu.org/licenses/>.                                        */

#ifndef ASMFFI_H
#define ASMFFI_H

#include "qubfast.h"
#define ASMFFI_API QUBFAST_API

#include <iostream>
#include "callbk_reportfun.h"
#ifdef EMSCRIPTEN
#include "callbk_reportstream.h"
extern callbk_reportstream console;
#else
#define console std::cerr
#endif

#ifdef __cplusplus
  extern "C" {
#endif

    double* ASMFFI_API dalloc(size_t n);
    void ASMFFI_API dfree(double *buf);
    double ASMFFI_API dget(double *buf, size_t i);
    void ASMFFI_API dset(double *buf, size_t i, double x);
    float* ASMFFI_API falloc(size_t n);
    void ASMFFI_API ffree(float *buf);
    float ASMFFI_API fget(float *buf, size_t i);
    void ASMFFI_API fset(float *buf, size_t i, float x);
    int* ASMFFI_API ialloc(size_t n);
    void ASMFFI_API ifree(int *buf);
    int ASMFFI_API iget(int *buf, size_t i);
    void ASMFFI_API iset(int *buf, size_t i, int x);
    long long ASMFFI_API i64get(short *buf, size_t i);
    void ASMFFI_API i64set(long long *buf, size_t i, long long x);
    short ASMFFI_API i16get(short *buf, size_t i);
    void ASMFFI_API i16set(short *buf, size_t i, short x);
    char ASMFFI_API i8get(char *buf, size_t i);
    void ASMFFI_API i8set(char *buf, size_t i, char x);
    void** ASMFFI_API palloc(size_t n);
    void ASMFFI_API pfree(void **buf);
    void* ASMFFI_API pget(void **buf, size_t i);
    void ASMFFI_API pset(void **buf, size_t i, void *x);
    int** ASMFFI_API ialloc2(size_t n, size_t m);
    void ASMFFI_API ifree2(int **bufs);
    float** ASMFFI_API falloc2(size_t n, size_t m);
    void ASMFFI_API ffree2(float **bufs);
    float* ASMFFI_API frow2(float **bufs, size_t i);
    double** ASMFFI_API dalloc2(size_t n, size_t m);
    void ASMFFI_API dfree2(double **bufs);
    double* ASMFFI_API drow2(double **bufs, size_t i);
    void ASMFFI_API wrap_memcpy(void *dest, void *src, size_t bytes);
    
    int ASMFFI_API consoleLog(const char *msg, void *data);    

#ifdef __cplusplus
  }
#endif

#endif
